useDynLib(ameras, .registration = TRUE)
importFrom(Rcpp, evalCpp)
import(RcppEigen)
import(methods)
importFrom(
  "stats",
  "cov",
  "var",
  "optim",
  "pchisq",
  "quantile",
  "rexp",
  "qnorm",
  "rnorm",
  "runif",
  "sd",
  "uniroot",
  "optimize",
  ".getXlevels",
  "model.frame",
  "model.matrix",
  "na.pass",
  "terms",
  "as.formula",
  "confint",
  "qqline",
  "qqnorm",
  "ppoints",
  "smooth.spline"
)
importFrom("mvtnorm", "rmvnorm")
importFrom(
  "nimble",
  "nimbleCode",
  "nimbleFunction",
  "getNimbleOption",
  "nf_preProcessMemberDataObject",
  "nimbleModel",
  "compileNimble",
  "configureMCMC",
  "buildMCMC",
  "runMCMC"
)
importFrom("coda", "HPDinterval", "as.mcmc")
importFrom("numDeriv", "hessian")
importFrom("MCMCvis", "MCMCsummary", "MCMCtrace")
importFrom("utils", "head")
importFrom("graphics", "abline", "panel.smooth", "par", "text", "lines")
importFrom("grDevices", "dev.interactive")
export(
  ameras,
  transform1,
  transform1.inv,
  transform1.jacobian,
  traceplot,
  ecdfplot,
  dose_lrt,
  Rhat,
  included_realizations,
  convergence,
  summary_table
)
S3method(coef, amerasfit)
S3method(summary, amerasfit)
S3method(print, summary.amerasfit)
S3method(traceplot, amerasfit)
S3method(print, amerasfit)
S3method(confint, amerasfit)
S3method(dose_lrt, amerasfit)
S3method(included_realizations, amerasfit)
S3method(convergence, amerasfit)
S3method(Rhat, amerasfit)
S3method(residuals, amerasfit)
S3method(plot, amerasfit)
S3method(summary_table, amerasfit)
S3method(vcov, amerasfit)
