# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

dldd_clogit <- function(designmat, RRs) {
    .Call(`_ameras_dldd_clogit`, designmat, RRs)
}

dldd_prophaz <- function(entry_t, exit_t, status_ord, RRs) {
    .Call(`_ameras_dldd_prophaz`, entry_t, exit_t, status_ord, RRs)
}

compute_ERCmatrix_prophaz <- function(entry_t, exit_t, status_ord, RRs, drdd, drdd2) {
    .Call(`_ameras_compute_ERCmatrix_prophaz`, entry_t, exit_t, status_ord, RRs, drdd, drdd2)
}

compute_ERCmatrix_clogit <- function(designmat, RRs, drdd, drdd2, status) {
    .Call(`_ameras_compute_ERCmatrix_clogit`, designmat, RRs, drdd, drdd2, status)
}

loglik_prophaz_rcpp <- function(exit_t, entry_t, RR_entry, RR_exit, status_ord, loglim) {
    .Call(`_ameras_loglik_prophaz_rcpp`, exit_t, entry_t, RR_entry, RR_exit, status_ord, loglim)
}

