# proflik/wald.transformed snapshot: RC

    Code
      fit1[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.69566358   0.03583481   0.27352443 

---

    Code
      fit1[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.08531710   0.20479501   0.07751758 

---

    Code
      fit1[[method]]$CI
    Output
                       lower     upper pval.lower pval.upper iter.lower iter.upper
      dose         0.0000000 0.4819361 0.86103023 0.05005070         NA         11
      dose_squared 0.1248245 0.3885281 0.04978527 0.04932973          7          7

---

    Code
      fit2[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.69566358   0.03583481   0.27352443 

---

    Code
      fit2[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.08531710   0.20479501   0.07751758 

---

    Code
      fit2[[method]]$CI
    Output
                           lower        upper
      (Intercept)  -8.628820e-01   -0.5284451
      dose          4.894522e-07 2623.6136590
      dose_squared  1.559819e-01    0.4746758

# proflik/wald.transformed snapshot: ERC

    Code
      fit1[[method]]$coefficients
    Output
        (Intercept)          dose  dose_squared 
      -6.934349e-01  5.339510e-08  3.150004e-01 

---

    Code
      fit1[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
      0.0525498471 0.0002282538 0.0469447230 

---

    Code
      fit1[[method]]$CI
    Output
                       lower     upper pval.lower pval.upper iter.lower iter.upper
      dose         0.0000000        NA 0.99231556         NA         NA         NA
      dose_squared 0.2316689 0.4201524 0.04930966 0.04946522          6          4

---

    Code
      fit2[[method]]$coefficients
    Output
        (Intercept)          dose  dose_squared 
      -6.934349e-01  5.339510e-08  3.150004e-01 

---

    Code
      fit2[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
      0.0525498471 0.0002282538 0.0469447230 

---

    Code
      fit2[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.7964307 -0.5904391
      dose          0.0000000        Inf
      dose_squared  0.2348924  0.4213916

# proflik/wald.transformed snapshot: MCML

    Code
      fit1[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
      -0.678139870  0.008365397  0.275598584 

---

    Code
      fit1[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.08413263   0.19771678   0.07481774 

---

    Code
      fit1[[method]]$CI
    Output
                       lower     upper pval.lower pval.upper iter.lower iter.upper
      dose         0.0000000 0.4419211 0.96622836 0.05007639         NA         14
      dose_squared 0.1314272 0.3809827 0.04983462 0.04945301          7          7

---

    Code
      fit2[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
      -0.678139870  0.008365397  0.275598584 

---

    Code
      fit2[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.08413263   0.19771678   0.07481774 

---

    Code
      fit2[[method]]$CI
    Output
                           lower         upper
      (Intercept)  -8.430368e-01 -5.132430e-01
      dose          6.371978e-23  1.098244e+18
      dose_squared  1.609797e-01  4.673852e-01

# percentile/hpd snapshot: FMA

    Code
      fit1[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.67942038   0.01295819   0.27406256 

---

    Code
      fit1[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.08348567   0.19530387   0.07436304 

---

    Code
      fit1[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.8447126 -0.5159928
      dose         -0.3644905  0.4016076
      dose_squared  0.1266253  0.4193271

---

    Code
      fit2[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.67942038   0.01295819   0.27406256 

---

    Code
      fit2[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.08348567   0.19530387   0.07436304 

---

    Code
      fit2[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.8477902 -0.5195303
      dose         -0.3662872  0.3992981
      dose_squared  0.1237895  0.4162422

# percentile/hpd snapshot: BMA

    Code
      fit1[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
        -0.7233563    0.1343364    0.2481227 

---

    Code
      fit1[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.10837398   0.24208244   0.06946021 

---

    Code
      fit1[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.9958492 -0.5341520
      dose         -0.2427882  0.6850701
      dose_squared  0.1285954  0.3998694

---

    Code
      fit2[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
        -0.7233563    0.1343364    0.2481227 

---

    Code
      fit2[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.10837398   0.24208244   0.06946021 

---

    Code
      fit2[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.9129171 -0.5102140
      dose         -0.2420483  0.6938121
      dose_squared  0.1054921  0.3744100

