# Poisson snapshot: RC

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.88451188   0.62049067  -0.04009246 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
       0.040752584  0.039808460  0.007371927 

---

    Code
      fit[[method]]$CI
    Output
                         lower       upper
      (Intercept)  -0.96438547 -0.80463828
      dose          0.54246753  0.69851382
      dose_squared -0.05454117 -0.02564375

# Poisson snapshot: ERC

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.87990056   0.61124972  -0.03787445 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
       0.040455983  0.038738306  0.007016268 

---

    Code
      fit[[method]]$CI
    Output
                         lower       upper
      (Intercept)  -0.95919283 -0.80060829
      dose          0.53532404  0.68717540
      dose_squared -0.05162609 -0.02412282

# Poisson snapshot: MCML

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.85434971   0.59374567  -0.03682549 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
       0.038539489  0.034503349  0.005806256 

---

    Code
      fit[[method]]$CI
    Output
                         lower       upper
      (Intercept)  -0.92988573 -0.77881370
      dose          0.52612035  0.66137099
      dose_squared -0.04820554 -0.02544543

# Poisson snapshot: FMA

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.85774451   0.59961048  -0.03827769 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
       0.040022104  0.039106177  0.007296265 

---

    Code
      fit[[method]]$CI
    Output
                         lower       upper
      (Intercept)  -0.93763984 -0.78033841
      dose          0.52798149  0.68502270
      dose_squared -0.05628895 -0.02592227

# Poisson snapshot: BMA

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared 
       -0.81604201   0.54829635  -0.03046823 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared 
        0.08482277   0.10231669   0.01742583 

---

    Code
      fit[[method]]$CI
    Output
                        lower        upper
      (Intercept)  -0.9362879 -0.628417092
      dose          0.3271861  0.703246734
      dose_squared -0.0575862  0.005196177

