# clogit snapshot: RC

    Code
      fit[[method]]$coefficients
    Output
              dose dose_squared 
        0.63693308  -0.04695187 

---

    Code
      fit[[method]]$sd
    Output
              dose dose_squared 
        0.09942130   0.02204514 

---

    Code
      fit[[method]]$CI
    Output
                         lower        upper
      dose          0.44207092  0.831795239
      dose_squared -0.09015955 -0.003744198

# clogit snapshot: ERC

    Code
      fit[[method]]$coefficients
    Output
              dose dose_squared 
        0.21121675   0.05299386 

---

    Code
      fit[[method]]$sd
    Output
              dose dose_squared 
       0.035301590  0.008925011 

---

    Code
      fit[[method]]$CI
    Output
                        lower      upper
      dose         0.14202690 0.28040659
      dose_squared 0.03550116 0.07048656

# clogit snapshot: MCML

    Code
      fit[[method]]$coefficients
    Output
              dose dose_squared 
         0.6680322   -0.0567844 

---

    Code
      fit[[method]]$sd
    Output
              dose dose_squared 
        0.09813158   0.02085082 

---

    Code
      fit[[method]]$CI
    Output
                         lower       upper
      dose          0.47569785  0.86036658
      dose_squared -0.09765124 -0.01591755

# clogit snapshot: FMA

    Code
      fit[[method]]$coefficients
    Output
              dose dose_squared 
        0.66550565  -0.05621879 

---

    Code
      fit[[method]]$sd
    Output
              dose dose_squared 
        0.10248403   0.02214376 

---

    Code
      fit[[method]]$CI
    Output
                         lower       upper
      dose          0.46040666  0.86355768
      dose_squared -0.09928857 -0.01171218

# clogit snapshot: BMA

    Code
      fit[[method]]$coefficients
    Output
              dose dose_squared 
        0.62957289  -0.04912569 

---

    Code
      fit[[method]]$sd
    Output
              dose dose_squared 
        0.13710969   0.03042399 

---

    Code
      fit[[method]]$CI
    Output
                         lower       upper
      dose          0.36396710 0.858500724
      dose_squared -0.09865736 0.006771486

