# gaussian snapshot: RC

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared        sigma 
        -0.8596448    0.4123961    0.1923798    1.1007480 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared        sigma 
        0.03544423   0.04824560   0.01132056   0.01421045 

---

    Code
      fit[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.9291143 -0.7901754
      dose          0.3178365  0.5069558
      dose_squared  0.1701919  0.2145677
      sigma         1.0728961  1.1286000

# gaussian snapshot: ERC

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared        sigma 
        -0.8575908    0.4073724    0.1937510    1.0998647 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared        sigma 
        0.03542220   0.04824927   0.01135011   0.01418433 

---

    Code
      fit[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.9270170 -0.7881645
      dose          0.3128056  0.5019393
      dose_squared  0.1715052  0.2159968
      sigma         1.0720639  1.1276655

# gaussian snapshot: MCML

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared        sigma 
        -0.8632862    0.4879678    0.1482892    1.1510453 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared        sigma 
        0.03670882   0.04979688   0.01130102   0.01485994 

---

    Code
      fit[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.9352342 -0.7913382
      dose          0.3903677  0.5855679
      dose_squared  0.1261396  0.1704388
      sigma         1.1219203  1.1801702

# gaussian snapshot: FMA

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared        sigma 
        -0.8637789    0.4907493    0.1469475    1.1511773 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared        sigma 
        0.03702730   0.05193649   0.01291181   0.01488525 

---

    Code
      fit[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.9365178 -0.7910689
      dose          0.3902604  0.5940566
      dose_squared  0.1183053  0.1707397
      sigma         1.1219463  1.1803065

# gaussian snapshot: BMA

    Code
      fit[[method]]$coefficients
    Output
       (Intercept)         dose dose_squared        sigma 
        -0.8607539    0.4838584    0.1478250    1.1509975 

---

    Code
      fit[[method]]$sd
    Output
       (Intercept)         dose dose_squared        sigma 
        0.03752324   0.04523186   0.01112508   0.01398801 

---

    Code
      fit[[method]]$CI
    Output
                        lower      upper
      (Intercept)  -0.9371247 -0.7902548
      dose          0.3954455  0.5666915
      dose_squared  0.1226427  0.1660678
      sigma         1.1228915  1.1756466

